@@ -26,7 +26,6 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"golang.org/x/net/context"
-	"google.golang.org/grpc"
 	"google.golang.org/grpc/codes"
 )
 
@@ -48,7 +47,7 @@ func TestRestartRootCA(t *testing.T) {
 
 	_, err := tc.NodeCAClients[0].NodeCertificateStatus(tc.Context, &api.NodeCertificateStatusRequest{NodeID: "foo"})
 	assert.Error(t, err)
-	assert.Equal(t, codes.NotFound, grpc.Code(err))
+	assert.Equal(t, codes.NotFound, testutils.ErrorCode(err))
 
 	tc.CAServer.Stop()
 	go tc.CAServer.Run(tc.Context)
@@ -57,7 +56,7 @@ func TestRestartRootCA(t *testing.T) {
 
 	_, err = tc.NodeCAClients[0].NodeCertificateStatus(tc.Context, &api.NodeCertificateStatusRequest{NodeID: "foo"})
 	assert.Error(t, err)
-	assert.Equal(t, codes.NotFound, grpc.Code(err))
+	assert.Equal(t, codes.NotFound, testutils.ErrorCode(err))
 }
 
 func TestIssueNodeCertificate(t *testing.T) {
@@ -318,9 +317,9 @@ func TestNewNodeCertificateRequiresToken(t *testing.T) {
 	)
 	assert.NoError(t, tc.MemoryStore.Update(func(tx store.Tx) error {
 		clusters, _ := store.FindClusters(tx, store.ByName(store.DefaultClusterName))
-		newWorkerToken = ca.GenerateJoinToken(&tc.RootCA)
+		newWorkerToken = ca.GenerateJoinToken(&tc.RootCA, false)
 		clusters[0].RootCA.JoinTokens.Worker = newWorkerToken
-		newManagerToken = ca.GenerateJoinToken(&tc.RootCA)
+		newManagerToken = ca.GenerateJoinToken(&tc.RootCA, false)
 		clusters[0].RootCA.JoinTokens.Manager = newManagerToken
 		return store.UpdateCluster(tx, clusters[0])
 	}))
