@@ -4,7 +4,6 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
-	"log"
 	"math/rand"
 	"net"
 	"os"
@@ -43,6 +42,8 @@ const (
 
 func init() {
 	store.WedgeTimeout = 3 * time.Second
+	grpclog.SetLoggerV2(grpclog.NewLoggerV2(ioutil.Discard, ioutil.Discard, ioutil.Discard))
+	logrus.SetOutput(ioutil.Discard)
 }
 
 var tc *cautils.TestCA
@@ -50,9 +51,6 @@ var tc *cautils.TestCA
 func TestMain(m *testing.M) {
 	tc = cautils.NewTestCA(nil)
 
-	grpclog.SetLogger(log.New(ioutil.Discard, "", log.LstdFlags))
-	logrus.SetOutput(ioutil.Discard)
-
 	// Set a smaller segment size so we don't incur cost preallocating
 	// space on old filesystems like HFS+.
 	wal.SegmentSizeBytes = 64 * 1024
@@ -796,10 +794,7 @@ func TestRaftUnreachableNode(t *testing.T) {
 	nodes[2].Server = s
 	raft.Register(s, nodes[2].Node)
 
-	go func() {
-		// After stopping, we should receive an error from Serve
-		assert.Error(t, s.Serve(wrappedListener))
-	}()
+	go s.Serve(wrappedListener)
 
 	raftutils.WaitForCluster(t, clockSource, nodes)
 	defer raftutils.TeardownCluster(nodes)
@@ -827,7 +822,7 @@ func TestRaftJoinWithIncorrectAddress(t *testing.T) {
 
 	err := n.JoinAndStart(context.Background())
 	assert.NotNil(t, err)
-	assert.Contains(t, grpc.ErrorDesc(err), "could not connect to prospective new cluster member using its advertised address")
+	assert.Contains(t, testutils.ErrorDesc(err), "could not connect to prospective new cluster member using its advertised address")
 
 	// Check if first node still has only itself registered in the memberlist
 	assert.Len(t, nodes[1].GetMemberlist(), 1)
@@ -980,12 +975,12 @@ func TestStreamRaftMessage(t *testing.T) {
 
 	raftMsg := &api.StreamRaftMessageRequest{Message: msg}
 	err = stream.Send(raftMsg)
-	assert.Error(t, err, "Received unexpected error EOF")
+	assert.NoError(t, err)
 
 	_, err = stream.CloseAndRecv()
-	errStr := fmt.Sprintf("grpc: received message length %d exceeding the max size %d", raftMsg.Size(), transport.GRPCMaxMsgSize)
+	errStr := fmt.Sprintf("grpc: received message larger than max (%d vs. %d)", raftMsg.Size(), transport.GRPCMaxMsgSize)
 	s, _ := status.FromError(err)
-	assert.Equal(t, codes.Internal, s.Code())
+	assert.Equal(t, codes.ResourceExhausted, s.Code())
 	assert.Equal(t, errStr, s.Message())
 
 	// Sending multiple snap messages with different indexes
@@ -1024,3 +1019,42 @@ func TestStreamRaftMessage(t *testing.T) {
 	errStr = fmt.Sprintf("Raft message chunk is not of type %d", raftpb.MsgSnap)
 	assert.Equal(t, errStr, s.Message())
 }
+
+// TestGetNodeIDByRaftID tests the GetNodeIDByRaftID function. It's a very
+// simple test but those are the kind that make a difference over time
+func TestGetNodeIDByRaftID(t *testing.T) {
+	t.Parallel()
+
+	nodes, _ := raftutils.NewRaftCluster(t, tc)
+	defer raftutils.TeardownCluster(nodes)
+
+	// get the member list
+	members := nodes[1].GetMemberlist()
+	// get all of the raft ids
+	raftIDs := make([]uint64, 0, len(members))
+	for _, member := range members {
+		raftIDs = append(raftIDs, member.RaftID)
+	}
+
+	// now go and get the nodeID of every raftID
+	for _, id := range raftIDs {
+		nodeid, err := nodes[1].GetNodeIDByRaftID(id)
+		assert.NoError(t, err, "raft ID %v should give us a node ID", id)
+		// now go through the member manually list and make sure this is
+		// correct
+		for _, member := range members {
+			assert.True(t,
+				// either both should match, or both should not match. if they
+				// are different, then there is an error
+				(member.RaftID == id) == (member.NodeID == nodeid),
+				"member with id %v has node id %v, but we expected member with id %v to have node id %v",
+				member.RaftID, member.NodeID, id, nodeid,
+			)
+		}
+	}
+
+	// now expect a nonexistent raft member to return ErrNoMember
+	id, err := nodes[1].GetNodeIDByRaftID(8675309)
+	assert.Equal(t, err, raft.ErrMemberUnknown)
+	assert.Empty(t, id)
+}
